==> cookie_bot_protection-1.0.2.tar.gz: cookie_bot_protection/src/EventSubscriber/RequestEventSubscriber.php <==
<?php

namespace Drupal\cookie_bot_protection\EventSubscriber;

use Drupal\cookie_bot_protection\Form\SettingsForm;
use Drupal\cookie_bot_protection\Routing\UncacheableLocalRedirectResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Manage the request events for the module.
 */
class RequestEventSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a RequestEventSubscriber object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The service to get the configurations.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $this->configFactory;
  }

  /**
   * Challenge to put a cookie and follow redirects.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   *
   * @return void
   *   Void.
   *
   * @SuppressWarnings(PHPMD.CyclomaticComplexity)
   * @SuppressWarnings(PHPMD.StaticAccess)
   * @SuppressWarnings(PHPMD.NPathComplexity)
   */
  public function challenge(RequestEvent $event): void {
    $salt = (string) Settings::get('hash_salt');
    $request = $event->getRequest();
    $config = $this->configFactory->get(SettingsForm::SETTINGS);
    $debug = $config->get('header_response_debug');
    $url_patterns = $config->get('url_protected_patterns');
    $redirect_error_delay = (int) $config->get('redirect_error_delay');

    if (empty($url_patterns) || !is_string($url_patterns)) {
      return;
    }

    $hash_headers = [
      'accept_language' => $request->getLanguages(),
      'user_agent' => $request->headers->get('User-Agent'),
      'ip' => $request->getClientIp(),
    ];

    $url_patterns = explode(PHP_EOL, $url_patterns);
    $pattern_found = FALSE;
    foreach ($url_patterns as $url_pattern) {
      $matches = [];
      $pcre_pattern = preg_replace('/\\\\u([a-fA-F0-9]{4})/', '\\x{\\1}', $url_pattern);
      $pcre_pattern = preg_replace('/\r$/', '', (string) $pcre_pattern);
      if ($pattern_found |= (preg_match('{' . $pcre_pattern . '}u', $request->getUri(), $matches) > 0)) {
        break;
      }
    }

    if (!$pattern_found) {
      return;
    }

    $hash_payload = json_encode($hash_headers);
    $checked_challenge = hash_hmac('sha256', $hash_payload . 'checked', $salt);
    $challenge = $event->getRequest()->cookies->get('drupal_cbp_challenge');
    $check_param = $request->query->get('drupal_cbp_check');
    if ($challenge !== $checked_challenge) {
      $unchecked_challenge = hash_hmac('sha256', $hash_payload . 'unchecked', $salt);
      if ($check_param !== "1") {
        $request->query->set('drupal_cbp_check', '1');
        $request->overrideGlobals();
        $cookie = Cookie::create('drupal_cbp_challenge', $unchecked_challenge);
        $response = new UncacheableLocalRedirectResponse($request->getUri());
        $response->headers->setCookie($cookie);
      }
      elseif ($challenge === $unchecked_challenge) {
        $request->query->remove('drupal_cbp_check');
        $request->overrideGlobals();
        $response = new UncacheableLocalRedirectResponse($request->getUri());
        $cookie = Cookie::create('drupal_cbp_challenge', $checked_challenge);
        $response->headers->setCookie($cookie);
      }
      else {
        $response = new Response(
          $this->t(
            "Forbidden, your browser must accept cookies or the url have a wrong parameter. You'll be redirected in %seconds seconds",
            ['%seconds' => 5]
          )->render(), 403);
        if ($redirect_error_delay > 0) {
          $request->query->remove('drupal_cbp_check');
          $request->overrideGlobals();
          $response->headers->set('Refresh', sprintf('%d; url=%s', $redirect_error_delay, $request->getUri()));
        }
        $response->headers->removeCookie('drupal_cbp_challenge');
        $request->overrideGlobals();
      }
    }
    elseif ($check_param !== NULL) {
      $request->query->remove('drupal_cbp_check');
      $request->overrideGlobals();
      $response = new UncacheableLocalRedirectResponse($request->getUri());
    }

    if (isset($response) && (($check_param !== NULL) || ($challenge !== $checked_challenge))) {
      if ($debug) {
        $response->headers->add([
          'Drupal-CBP-Debug' => 'TRUE',
          'Drupal-CBP-Date' => (new DrupalDateTime())->format('c.v'),
          'Drupal-CBP-Request-Challenge' => $challenge,
          'Drupal-CBP-Unchecked-Challenge' => $unchecked_challenge ?? '',
          'Drupal-CBP-Checked-Challenge' => $checked_challenge,
          'Drupal-CBP-Checked-Query' => $check_param,
          'Drupal-CBP-Request-Headers' => $hash_payload,
        ]);
      }

      $this->prepareResponse($response);
      $event->setResponse($response);
      $event->stopPropagation();
    }
  }

  /**
   * Prepare the redirect response with non-cacheability.
   *
   * @param \Symfony\Component\HttpFoundation\Response $response
   *   The response to prepare.
   *
   * @return void
   *   Void
   */
  protected function prepareResponse(Response &$response): void {
    $response->setPrivate();
    $response->setMaxAge(0);
    $response->setSharedMaxAge(0);
    $response->setVary('Cookie');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => ['challenge', 1000],
    ];
  }

}

==> cookie_bot_protection-1.0.3.tar.gz: cookie_bot_protection/src/EventSubscriber/RequestEventSubscriber.php <==
<?php

namespace Drupal\cookie_bot_protection\EventSubscriber;

use Drupal\cookie_bot_protection\Form\SettingsForm;
use Drupal\cookie_bot_protection\Routing\UncacheableLocalRedirectResponse;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Site\Settings;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Manage the request events for the module.
 */
class RequestEventSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a RequestEventSubscriber object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The service to get the configurations.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Challenge to put a cookie and follow redirects.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   *
   * @return void
   *   Void.
   *
   * @SuppressWarnings(PHPMD.CyclomaticComplexity)
   * @SuppressWarnings(PHPMD.StaticAccess)
   * @SuppressWarnings(PHPMD.NPathComplexity)
   */
  public function challenge(RequestEvent $event): void {
    $salt = (string) Settings::get('hash_salt');
    $request = $event->getRequest();
    $config = $this->configFactory->get(SettingsForm::SETTINGS);
    $debug = $config->get('header_response_debug');
    $url_patterns = $config->get('url_protected_patterns');
    $redirect_error_delay = (int) $config->get('redirect_error_delay');

    if (empty($url_patterns) || !is_string($url_patterns)) {
      return;
    }

    $hash_headers = [
      'accept_language' => $request->getLanguages(),
      'user_agent' => $request->headers->get('User-Agent'),
      'ip' => $request->getClientIp(),
    ];

    $url_patterns = explode(PHP_EOL, $url_patterns);
    $pattern_found = FALSE;
    foreach ($url_patterns as $url_pattern) {
      $matches = [];
      $pcre_pattern = preg_replace('/\\\\u([a-fA-F0-9]{4})/', '\\x{\\1}', $url_pattern);
      $pcre_pattern = preg_replace('/\r$/', '', (string) $pcre_pattern);
      if ($pattern_found |= (preg_match('{' . $pcre_pattern . '}u', $request->getUri(), $matches) > 0)) {
        break;
      }
    }

    if (!$pattern_found) {
      return;
    }

    $hash_payload = json_encode($hash_headers);
    $checked_challenge = hash_hmac('sha256', $hash_payload . 'checked', $salt);
    $challenge = $event->getRequest()->cookies->get('drupal_cbp_challenge');
    $check_param = $request->query->get('drupal_cbp_check');
    if ($challenge !== $checked_challenge) {
      $unchecked_challenge = hash_hmac('sha256', $hash_payload . 'unchecked', $salt);
      if ($check_param !== "1") {
        $request->query->set('drupal_cbp_check', '1');
        $request->overrideGlobals();
        $cookie = Cookie::create('drupal_cbp_challenge', $unchecked_challenge);
        $response = new UncacheableLocalRedirectResponse($request->getUri());
        $response->headers->setCookie($cookie);
      }
      elseif ($challenge === $unchecked_challenge) {
        $request->query->remove('drupal_cbp_check');
        $request->overrideGlobals();
        $response = new UncacheableLocalRedirectResponse($request->getUri());
        $cookie = Cookie::create('drupal_cbp_challenge', $checked_challenge);
        $response->headers->setCookie($cookie);
      }
      else {
        $response = new Response(
          $this->t(
            "Forbidden, your browser must accept cookies or the url have a wrong parameter. You'll be redirected in %seconds seconds",
            ['%seconds' => 5]
          )->render(), 403);
        if ($redirect_error_delay > 0) {
          $request->query->remove('drupal_cbp_check');
          $request->overrideGlobals();
          $response->headers->set('Refresh', sprintf('%d; url=%s', $redirect_error_delay, $request->getUri()));
        }
        $response->headers->removeCookie('drupal_cbp_challenge');
        $request->overrideGlobals();
      }
    }
    elseif ($check_param !== NULL) {
      $request->query->remove('drupal_cbp_check');
      $request->overrideGlobals();
      $response = new UncacheableLocalRedirectResponse($request->getUri());
    }

    if (isset($response) && (($check_param !== NULL) || ($challenge !== $checked_challenge))) {
      if ($debug) {
        $response->headers->add([
          'Drupal-CBP-Debug' => 'TRUE',
          'Drupal-CBP-Date' => (new DrupalDateTime())->format('c.v'),
          'Drupal-CBP-Request-Challenge' => $challenge,
          'Drupal-CBP-Unchecked-Challenge' => $unchecked_challenge ?? '',
          'Drupal-CBP-Checked-Challenge' => $checked_challenge,
          'Drupal-CBP-Checked-Query' => $check_param,
          'Drupal-CBP-Request-Headers' => $hash_payload,
        ]);
      }

      $this->prepareResponse($response);
      $event->setResponse($response);
      $event->stopPropagation();
    }
  }

  /**
   * Prepare the redirect response with non-cacheability.
   *
   * @param \Symfony\Component\HttpFoundation\Response $response
   *   The response to prepare.
   *
   * @return void
   *   Void
   */
  protected function prepareResponse(Response &$response): void {
    $response->setPrivate();
    $response->setMaxAge(0);
    $response->setSharedMaxAge(0);
    $response->setVary('Cookie');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => ['challenge', 1000],
    ];
  }

}
